package listFW.visitor;
import listFW.*;

/**
 * Computes a String reprsentation of IList showing a left parenthesis followed
 * by elements of the IList separated by commas, ending with with a right parenthesis.
 * Non-tail recursive version.
 * @stereotype visitor
 * @author D.X. Nguyen
 * @since 03/08/2004
 */
public class ToString3 implements IListAlgo {
    
    public static final ToString3 Singleton = new ToString3();
    private ToString3() {
    }
    
    /**
     * Returns "()".
     */
    public Object emptyCase(IMTList host, Object...nu) {
        return "()";
    }
    
    /**
     * Passes "(" + first to the rest of IList and 
     * asks for help to complete the computation.
     */
    public Object nonEmptyCase(INEList host, Object...nu) {
        return "(" + host.getFirst() + host.getRest().execute(ToString3Help.Singleton);
    }
}

/**
 * Helps ToString3 compute the String representation of the rest of the list.
 */
class ToString3Help implements IListAlgo {
    public static final ToString3Help Singleton = new ToString3Help();
    private ToString3Help() {
    }
    
    /**
     * Returns ")".
     * At end of list: done!  
     */
    public Object emptyCase(IMTList host, Object... nu) {
        return ")";
    }
    
    /**
     */
    public Object nonEmptyCase(INEList host, Object... nu) {
        return ", " + host.getFirst() + host.getRest().execute(this);
    }
}

